//favouriteslistview.cpp

#include <qstring.h>
#include <qtooltip.h>
#include <qinputdialog.h>
#include <qfile.h>
#include <qtextstream.h>
#include <stdlib.h>

#include "favouriteslistview.h"
#include "settingsdialog.h"

FavouritesListView::FavouritesListView(QWidget * parent, const char * name, WFlags f ) : QListView( parent, name, f )
{
  this->addColumn( "Description" );
  this->addColumn( "Hostname" );
  this->addColumn( "Username" );
  this->addColumn( "Portnumber" );
  this->setRootIsDecorated( true );
  this->setShowToolTips( true );
}

void FavouritesListView::setReferens( SettingsDialog *ref )
{
  _referens = ref;
  QObject::connect( this, SIGNAL( doubleClicked( QListViewItem *) ), _referens, SLOT( slot_displayInfo( QListViewItem * ) ) ); 
}

void FavouritesListView::load()
{

  QString strGenre, strHostName, strUserName, strPortNumber, strNickName, buff;
  QFile f( _path ); 
  if ( !f.open( IO_ReadOnly ) ) {
    return;
  }
  QTextStream t( (QIODevice*)&f );
  
  while ( buff = t.readLine() )
    {
      if ( (buff[0] == '[') && (buff[buff.length() - 1] == ']') )
	{	
	  if ( strGenre != buff.mid( 1, buff.find("\\") - 1 ) )
	    {     
	      strGenre = buff.mid( 1, buff.find("\\") - 1 );
	      addGenre( strGenre );
	    }
	  strNickName = buff.mid( buff.find("\\") + 1, buff.length() - buff.find("\\") - 2 );
	  
	  buff = t.readLine();
	  
	  if( buff.mid(0, 9) == "hostname=" )
	    strHostName = buff.mid( 9, buff.length() - 9 );

	  buff = t.readLine();
	  if( buff.mid(0, 9) == "username=" )
	    strUserName = buff.mid( 9, buff.length() - 9);

	  buff = t.readLine();
	  if( buff.mid(0, 11) == "portnumber=" )
	    strPortNumber = buff.mid( 11, buff.length() - 11);
	   
	  addHostName( strGenre, strHostName, strNickName, strUserName, strPortNumber );
	}
    }
  
  f.close();
}

void FavouritesListView::save()
{
  QFile f( _path ); 
  if ( !f.open( IO_WriteOnly ) )
    return;
  QTextStream t( (QIODevice*)&f );
  t << "/**************************************************\\\n"
       "* Favourite handler file for jFtp                  *\n"
       "* Authors: Joe Paulsson <joepaulsson@hotmail.com>  *\n"
       "*          Anders Pettersson <d00_ape@hotmail.com> *\n"
       "*          Jesper Miller <miller@linux.nu>         *\n"
       "*          Mattias Gustafsson <matteg@linux.nu>    *\n"
       "*	   David Edstrm <fallen@mangakai.org>	   *\n"
      "\\**************************************************/\n" << endl;
  
   QListViewItemIterator it( this );
   QString strGenre, strNickName;
   while ( it.current() )
   {
       if ( !it.current()->parent() )
	 strGenre = it.current()->text(0);

       ++it;
       if ( it.current() && it.current()->parent() )
	 {
	   while ( it.current() && it.current()->parent() )
	     {
	   
	       t << "\n[" << strGenre << "\\" << it.current()->text(0) << "]" << endl;
	       t << "hostname=" << it.current()->text(1) << endl;
	       t << "username=" << it.current()->text(2) << endl;
	       t << "portnumber=" << it.current()->text(3) << endl;
	       ++it;   
	     }
	 }
   }
  f.close();
}

void FavouritesListView::setPath( QString p )
{
  _path = p + ".favourites.txt"; 
}

void FavouritesListView::addHostName()
{
  bool ok = false;
  if ( this->selectedItem() )
    {
      QListViewItem *tmp = this->selectedItem();
      if ( tmp->parent() )
	tmp = tmp->parent();
 
      QString text2 = QInputDialog::getText(
	              "Add hostname in " + tmp->text(0),
                      "Please enter hostname:",
                      QLineEdit::Normal, QString::null, &ok, _referens );
      if ( ok && !text2.isEmpty() )
	{
	  QString text1 = QInputDialog::getText(
	                  "Description of " + text2 ,
                          "Please enter description of " + text2 + ":",
                          QLineEdit::Normal, QString::null, &ok, _referens );
   
	  if( ok )
	    {
	      QString text3 = QInputDialog::getText(
	                      "Username for " + text2 ,
                              "Please enter username for " + text2 + ":",
                              QLineEdit::Normal, QString::null, &ok, _referens );
	      if( ok )
		{
		  QString text4;
		  text4.setNum(QInputDialog::getInteger(
		                  "Portnumber for " + text2 ,
                                  "Please enter portnumber for " + text2 + ":",
                                  21, 0, 65530, 1, &ok, _referens ) ); 
		  if( ok )
		    tmp = new QListViewItem( tmp, text1, text2, text3, text4 );
		}
	    }
	}
    } 
}

void FavouritesListView::addHostName(const QString genre, const QString hostname, const QString nickname, const QString username, const QString password)
{

      QListViewItemIterator it( this );
      bool f = true;
      while ( f )
	{
	  if ( it.current()->text(0) == genre && !it.current()->parent() ) 
	    f = false;
	  else
	    ++it;
	}
      QListViewItem *g = it.current();
      g = new QListViewItem( g, nickname, hostname, username, password );   
}


void FavouritesListView::addGenre()
{
  bool ok = FALSE;
  QString text = QInputDialog::getText(
	          "Add genre" ,
                 "Please enter genrename:",
                 QLineEdit::Normal, QString::null, &ok, _referens );
  if ( ok && !text.isEmpty() )
    {
      QListViewItem *genre = new QListViewItem( this );
      genre->setText( 0, text );
    }
  else
    ;

}

void FavouritesListView::addGenre( const QString g)
{
  QListViewItem *genre = new QListViewItem( this );
  genre->setText( 0, g );
}

void FavouritesListView::erase()
{
  QListViewItem *g = this->selectedItem();
  if( !g )
    return;
 delete g; 
}

