//filetransferlistview.cpp

#include <qlistview.h>
#include <qstring.h>
#include <qvaluelist.h>

#include "filetransferlistview.h" 
#include "filetransferlistviewitem.h"  

FileTransferListView::FileTransferListView( QWidget *parent, const char *name  ) : QListView( parent, name )
{
  addColumn( "File" );
  addColumn( "Procent" );
  addColumn( "Speed" );
  addColumn( "Bytes done" );
  addColumn( "Elapsed time" );
  addColumn( "Time left" );
  addColumn( "Finish time" );
  setGeometry(10, 10, 300, 200);
  setMultiSelection(true);
  setSelectionMode(QListView::Extended);
  setColumnAlignment(1, Qt::AlignCenter);
  setColumnAlignment(2, Qt::AlignCenter);
  setColumnAlignment(3, Qt::AlignCenter);
  setColumnAlignment(4, Qt::AlignCenter);
  setColumnAlignment(5, Qt::AlignCenter);
  setColumnAlignment(6, Qt::AlignCenter);
  setShowSortIndicator(TRUE);
  setAllColumnsShowFocus(TRUE);
  setSelectionMode(Extended);
  setSorting( -1 );

  connect( this, SIGNAL( rightButtonPressed(QListViewItem *, const QPoint &, int) ), this, SLOT( slotRightPressed( QListViewItem *, const QPoint &, int) ) );
}

void FileTransferListView::slotInsertItem( const QString &filename )
{
  FileTransferListViewItem *item = new FileTransferListViewItem( this, filename, 0 );
  itemList.push_front( item );
  insertItem( item ); 
}

void FileTransferListView::slotUppdateItem( const QString &filename, int bytesDone, int bytesTotal )
{ 
  QValueListIterator<FileTransferListViewItem *> iter;
  if( !itemList.isEmpty() )
  {
    int i=0;
    for( iter = itemList.begin(); iter != itemList.end(); iter++ )
    {
      if( (*iter)->getName() == filename && (*iter)->getState() != Finished )	
      {
	itemList[i]->updateInfo( bytesDone, bytesTotal );
	return;
      }
      i++;
    }
  }
}

void FileTransferListView::slotFileError( const QString &filename, const QString &error )
{
  QValueListIterator<FileTransferListViewItem *> iter;
  if( !itemList.isEmpty() )
  {
    int i=0;
    for( iter = itemList.begin(); iter != itemList.end(); iter++ )
    {
      if( (*iter)->getName() == filename )	
      {
	itemList[i]->setFailed( error );
	return;
      }
      i++;
    }
  }
}


void FileTransferListView::slotRightPressed(QListViewItem *listItem, const QPoint &point, int i)
{
  listItem = 0;
  i = 0;
  FileTransferListViewPopup *pop  = new FileTransferListViewPopup(this,"popupMenu");
  pop->popup(point);
}

void FileTransferListView::slotClearFinished()
{
  QValueListIterator<FileTransferListViewItem *> iter;
  if( !itemList.isEmpty() )
    for( iter = itemList.begin(); iter != itemList.end(); iter++ )
      if( (*iter)->getState() == Finished )	
	takeItem( (*iter) );
}

void FileTransferListView::slotClearAllStoppedTransfers()
{
  QValueListIterator<FileTransferListViewItem *> iter;
  if( !itemList.isEmpty() )
    for( iter = itemList.begin(); iter != itemList.end(); iter++ )
      if( (*iter)->getState() == Running ) 
	return;
  for( iter = itemList.begin(); iter != itemList.end(); iter++ )
    takeItem( (*iter) );
}

void FileTransferListView::slotClear() {
  clear();
  itemList.clear();
}

void FileTransferListView::slotStopTransfer() {
  QValueListIterator<FileTransferListViewItem *> iter;
  int i=0;
  for( iter = itemList.begin(); iter != itemList.end(); iter++ )
  {
    if( (*iter)->getState() == Running ) 
    {
      itemList[i]->setFailed( "STOPPED" );
      return;
    }
    i++;
  }
  emit signalStopTransfer();
}

// ************************** FileTransferListViewPopup ****************************// 

FileTransferListViewPopup::FileTransferListViewPopup(QWidget *parent, const char *name) : QPopupMenu(parent, name)
{
  insertItem("Clear Finished", parent, SLOT( slotClearFinished() ) );
  insertItem("Clear all stopped transfers", parent, SLOT( slotClearAllStoppedTransfers() ) );
  insertItem("Stop current transfer", parent, SLOT(slotStopTransfer()));
  insertSeparator(2); 
}
