//filetransferlistviewitem.cpp

#include <qlistview.h>
#include <qstringlist.h>
#include <qdatetime.h>
#include <qtimer.h> 

#include "filetransferlistviewitem.h"

FileTransferListViewItem::FileTransferListViewItem( QListView * parent, QString filename, int procent, QString speed, QString bytesDone, QString elapsedTime, QString timeLeft, QString finishtime )
   : QListViewItem( parent, filename, (QString::number( procent ) + "%"), speed, bytesDone, elapsedTime, timeLeft, finishtime )
{
  _name = filename;
  _state = NotStarted;
  _t.start(); 
}
                               
void FileTransferListViewItem::updateInfo( int bytesDone, int bytesTotal )
{
  float ms = _t.elapsed();
  float procentDone = 100 * ( (float)bytesDone / (float)bytesTotal ) + 0.5;
  float bytespersec = (float)bytesDone / ms;
  _state = Running;
  
  // Procent
  setText( 1, QString::number( (int)procentDone ) + "%" );
  
  // Speed
  setText( 2, QString::number( (int)bytespersec ) + " kb/s" );
  
  // Bytes Done
  
  setText( 3, QString::number( (int)bytesDone / 1000 ) + "kb / " + QString::number( (int)bytesTotal / 1000)  + "kb" );
  
  // Elapsed time
  QTime et(0,0,0);
  et = et.addMSecs( (int)ms );
  setText( 4, et.toString() );

  // Time left
  int bytesleft = bytesTotal - bytesDone;
  int mstoend = (int)((float)bytesleft / bytespersec);
  QTime ct(0,0,0);
  if( bytesDone == bytesTotal )
  {
    _state = Finished;
    setText( 5, "Finished" );
  } 
  else
  {         
    ct = ct.addMSecs( mstoend );
    setText( 5, ct.toString() );
  }

  // Finish time
  ct = QTime::currentTime(); // current time default
  ct = ct.addMSecs( mstoend );
  setText( 6, ct.toString() );

  
}

void FileTransferListViewItem::setFailed( const QString &error )
{
  setText( 5, "ERROR" );
  setText( 6, "< " + error + " >" );
  _state = Finished;
}

bool FileTransferListViewItem::equals( FileTransferListViewItem *item)
{
  if( QString::compare( item->getName(), getName() ) == 0)
    return TRUE;
  return FALSE;
}
  

