/*FileViewItem.cpp*/

#include <qpixmap.h>
#include "fileviewitem.h"

FileViewItem::FileViewItem(QListView *parent, const QUrlInfo &i) : QListViewItem(parent, i.name(), QString::number(i.size()), i.lastModified().toString() ), info(i)
{
}

FileViewItem::FileViewItem(QListView *parent, const QString n) : QListViewItem(parent, n)
{
}

int FileViewItem::compare(QListViewItem * i, int col, bool ascending ) const
{
  ascending = 0;
  FileViewItem *other = (FileViewItem*)i;
  switch (col) {
  case 0: {
    if(info.isDir() && other->entryInfo().isFile())
      return -1;
    else if(info.isFile() && other->entryInfo().isDir())
      return 1;
    else if(info.isDir() && other->entryInfo().isDir())
      return info.name() < other->entryInfo().name() ? -1 : 1;
    else if(info.isFile() && other->entryInfo().isFile())
      return info.name() < other->entryInfo().name() ? -1 : 1;
    break; 
  } 
  case 1: {
    if (info.size() == other->info.size())
      return 0;
    else
      return info.size() < other->info.size() ? -1 : 1;
  }
  case 2: {
    if (info.lastModified() == other->info.lastModified() )
      return 0;
    else
      return info.lastModified() < other->info.lastModified() ? -1 : 1;
  }
  default:
    return QListViewItem::compare( i, col, ascending );
  }
  return 0;
}

// foldericon XPM
static const char* folder_xpm[]={
  "15 15 6 1",
  ". c None",
  "b c #ffff00",
  "d c #000000",
  "* c #999999",
  "a c #cccccc",
  "c c #ffffff",
  "...............",
  "..*****........",
  ".*ababa*.......",
  "*abababa******.",
  "*cccccccccccc*d",
  "*cbababababab*d",
  "*cabababababa*d",
  "*cbababababab*d",
  "*cabababababa*d",
  "*cbababababab*d",
  "*cabababababa*d",
  "*cbababababab*d",
  "**************d",
  ".dddddddddddddd",
  "..............."};

// fileicon XPM
static const char* file_xpm[]={
  "13 15 5 1",
  ". c #7f7f7f",
  "# c None",
  "c c #000000",
  "b c #bfbfbf",
  "a c #ffffff",    //#ffffff
  "..........###",
  ".aaaaaaaab.##",
  ".aaaaaaaaba.#",
  ".aaaaaaaacccc",
  ".aaaaaaaaaabc",
  ".aaaaaaaaaabc",
  ".aaaaaaaaaabc",
  ".aaaaaaaaaabc",
  ".aaaaaaaaaabc",
  ".aaaaaaaaaabc",
  ".aaaaaaaaaabc",
  ".aaaaaaaaaabc",
  ".aaaaaaaaaabc",
  ".bbbbbbbbbbbc",
  "ccccccccccccc"};


const QPixmap *FileViewItem::pixmap(int col) const
{
  static QPixmap *folderIcon = 0;
  static QPixmap *fileIcon = 0;  
  
  if (!folderIcon)
    folderIcon = new QPixmap(folder_xpm);
  if (!fileIcon)
    fileIcon = new QPixmap(file_xpm);
  if (info.isDir() && col == 0)
    return folderIcon;
  else if (info.isFile() && col == 0)
    return fileIcon;
  return 0;
}



