//fileviewwidget.cpp

#include <qapplication.h>
#include <qpushbutton.h>
#include <qurloperator.h>
#include <qstring.h>
#include <qnetwork.h>
#include <qvaluelist.h>
#include <qnetworkprotocol.h>
#include <qlistview.h>
#include <qvbox.h>
#include <qcombobox.h>
#include <qinputdialog.h>
#include <qdialog.h>
#include <qmessagebox.h>
#include <qptrlist.h> 
#include <stdlib.h>
#include <unistd.h>
#include <qtooltip.h>

#include "fileviewwidget.h"
#include "fileviewitem.h"
#include "connectioninformationtext.h"
#include "errorandpic.h"
#include "ftpbase.h"



FileWidget::FileWidget(QWidget *parent, const char *name, FileWidget *other) : QVBox(parent, name), otherFw(other)
{
  qInitNetworkProtocols();
  init();
  QHBox *hbox = new QHBox( this );
  QPushButton *dirUp = new QPushButton("../", hbox, "dirUpButton");
  dirUp->setFixedWidth ( 25 );
  dirUp->setGeometry(5,5,40,25);
  QToolTip::add(dirUp, "One dir up in the filehierarchy");
  
  comboBox = new QComboBox(TRUE, hbox);
  comboBox->setGeometry(45,45,350,25);
  connect(comboBox, SIGNAL(activated(const QString &)), this,SLOT(slotComboBoxEnter(const QString &)));
  comboBox->insertItem(url.path(), 0);
  comboBox->setCurrentItem(0);
  comboBox->setMaxCount(5);
  comboBox->setDuplicatesEnabled(FALSE);
  comboBox->setInsertionPolicy(QComboBox::AtTop);

  fileView = new FileView(this);
  fileView->setGeometry(0, 40, 400, 400);

  /*Connect*/
  connect(fileView, SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(slotChangeDir(QListViewItem*)));
  connect(fileView, SIGNAL(returnPressed(QListViewItem*)), this, SLOT(slotChangeDir(QListViewItem*)));
  connect(fileView, SIGNAL(rightButtonPressed(QListViewItem *, const QPoint &, int)), this, SLOT(slotRightPressed(QListViewItem *, const QPoint &, int)));

  connect(&url, SIGNAL(start(QNetworkOperation*)), this, SLOT(slotStart(QNetworkOperation*)));
  connect(&url, SIGNAL(newChildren(const QValueList<QUrlInfo>&, QNetworkOperation*)), fileView, SLOT(slotInsert(const QValueList<QUrlInfo>&)));
  connect( &url, SIGNAL( dataTransferProgress( int, int, QNetworkOperation *) ), this, SLOT(slotUppdateTransfer( int, int, QNetworkOperation *) ) );
  connect( &url, SIGNAL( start( QNetworkOperation * ) ), this, SLOT( slotStart( QNetworkOperation * ) ) );
  connect( &url, SIGNAL( finished( QNetworkOperation * ) ), this, SLOT( slotFinished( QNetworkOperation * ) ) ); 
  connect(&url, SIGNAL(connectionStateChanged(int, const QString &)), this, SLOT(slotStateChanged(int, const QString&)));

  connect(dirUp, SIGNAL(clicked()), this, SLOT(slotDirUp()));
  connect(this, SIGNAL(signal_nextToCopy()), this, SLOT(slot_nextToCopy()));
  
  url.listChildren();
}

void FileWidget::init() 
{
  home = getenv("HOME");
  url = home;
  oldUrl = url;
}

char* FileWidget::getEnvHome() {
  return home;
}

void FileWidget::setReferens( ConnectionInformationText *ref )
{
  _referens = ref;
}

void FileWidget::setOtherFw(FileWidget *other) {
  otherFw = other;
}

QUrlOperator FileWidget::adress() {
  return url;
}

void FileWidget::setWidgetParent(FtpBase *adr) {
  widgetParent = adr;
}

void FileWidget::connectToFtp(QString &hostname, QString &username, QString &password, int port) {
  if(!hostname.isEmpty()) {
    _referens->addConnectionInformation("Connecting...", blue);
    QString host = "ftp://" + hostname;
    url = host;
    if(username.isEmpty())
      username = '\0';
    url.setUser(username);
    if(password.isEmpty())
      password = '\0';
    url.setPassword(password);
    url.setPort(port);
    url.setPath("/");
    comboBox->insertItem(url.path(), 0);
    comboBox->setCurrentItem(0);
    url.listChildren();    
  }
}

//*******SLOTS********
void FileWidget::slotDirUp() {
  if(url.cdUp()) {
    url.listChildren();
    comboBox->insertItem(url.path(), 0);
    comboBox->setCurrentItem(0);
  }
}

void FileWidget::slotStart(QNetworkOperation *op) {
  if ( !op )
    return;
  if ( op->operation() == QNetworkProtocol::OpListChildren )//Clear list 
    fileView->clear(); 
}

void FileWidget::slotStateChanged(int i, const QString &str) {
  i = 0;
  _referens->addConnectionInformation( str, red );
}


void FileWidget::slotDisconnect() {
  url.setHost("(null)");
  url.setProtocol("file");
  url.setPort(-1);
  url.stop();
  url.setPath("/");
  comboBox->insertItem(url.path(), 0);
  comboBox->setCurrentItem(0);  
  fileView->clear();
  widgetParent->changeBtnState(true);
  _referens->addConnectionInformation("Disconnected from server", red);
}

void FileWidget::slotChangeDir(QListViewItem *item) {
  FileViewItem *fileItem = (FileViewItem*) item;
  oldUrl = url;
  if(fileItem->entryInfo().isDir()) {
    if(fileItem->entryInfo().name() == "/") {
      url.setPath("/");
      url.listChildren();
    }
    else {
      url.addPath(fileItem->entryInfo().name());
      if(url.isLocalFile()) {
	QString unreadDir = fileItem->entryInfo().name();
	if(url.dirPath() == "/") {
	  url = oldUrl;
	  QString mess = "Couldn't read dir: ";
	  mess += url.path();
	  mess += unreadDir;
	  _referens->addConnectionInformation(mess, red);
	}
	else {
	  url.listChildren();
	  comboBox->insertItem(url.path(), 0);
	  comboBox->setCurrentItem(0);
	}	
      }
      else {
	url.listChildren();
	comboBox->insertItem(url.path(), 0);
	comboBox->setCurrentItem(0);
      }
    }	    
  }
}

void FileWidget::slotComboBoxEnter(const QString &path) {
  oldUrl = url;
  if(!url.hasPath())
    url = path;
  else
    url.setPath(path);  
  url.listChildren();
}

void FileWidget::slot_nextToCopy() {
  if(!selectedFiles.isEmpty()) {
    emit signalNewItem( selectedFiles.first().name() );
    url.copy((QUrl(url, (selectedFiles.first()).name())), otherFw->adress(), FALSE);
    selectedFiles.pop_front();
  }
  else
    widgetParent->changeBtnState(true);
}

void FileWidget::slotUpDownload() {
  selectedFiles = fileView->selectedItems();
  if(selectedFiles.isEmpty()) {
    _referens->addConnectionInformation("No files selected for upload/download", blue);
    widgetParent->changeBtnState(true);
    return;    
  }
  widgetParent->changeBtnState(false);
  slot_nextToCopy();
}

void FileWidget::slotRightPressed(QListViewItem *listItem, const QPoint &point, int i) {
  i = 0;
  listItem = 0;
  FileViewPopup *pop  = new FileViewPopup(this,"popupMenu");
  pop->popup(point);
}

void FileWidget::slotRefresh() {
  fileView->clear();
  url.listChildren();
}

void FileWidget::slotMarkAll() {
  fileView->selectAll(TRUE);
}

void FileWidget::slotMakedir() {
  bool ok = FALSE;
  QString name = QInputDialog::getText( tr( "Make new directory:" ), "Enter new directoryname", QLineEdit::Normal, QString::null, &ok, this );
  if(ok && !name.isEmpty()) {
    url.mkdir(name);
    slotRefresh();
  }
}

void FileWidget::slotDelete() {
  QValueList<QUrlInfo> delList = fileView->selectedItemsForDel();
  if(delList.isEmpty()) {
    _referens->addConnectionInformation("Nothing to delete", blue);
    return;
  }
  QValueList<QUrlInfo>::Iterator iter = delList.begin();
  bool all = false;
  for(; iter != delList.end(); ++iter) {
    if(!all) {
      int val = QMessageBox::information(this, "Delete confirm", "Delete: " + (*iter).name(), "yes", "all", "cancel");
      if(val == 0)
	url.remove((*iter).name());
      if(val == 1)
	all = true;
      if(val == 2)
	break;
    }
    if(all)
      url.remove((*iter).name());      
  }
  slotRefresh();
}

void FileWidget::slotRename() {
  QValueList<QUrlInfo> renameList = fileView->selectedItems();
  if(renameList.isEmpty()) {
    _referens->addConnectionInformation("Nothing to rename", blue);
    return;
  }
  bool ok = FALSE;
  QValueList<QUrlInfo>::Iterator iter = renameList.begin();
  for(; iter != renameList.end(); ++iter) {  
    QString newName = QInputDialog::getText( tr( "Rename" ), "Rename: " + (*iter).name(), QLineEdit::Normal, QString::null, &ok, this );
    if(ok && !newName.isEmpty()) 
      url.rename((*iter).name(), newName);  
  } 
  slotRefresh();
}
  
void FileWidget::slotUppdateTransfer( int bytesDone, int bytesTotal, QNetworkOperation * op)
{
  int i = op->arg(0).findRev( "/" );
  QString sBuff = op->arg(0).right( op->arg(0).length() - i - 1 );
  emit signalTransferInfo( sBuff, bytesDone, bytesTotal);
}

void FileWidget::slotStopTransfer() {
  url.stop();
  slot_nextToCopy();
}

void FileWidget::slotFinished(QNetworkOperation *op)
{
  if ( !op )
    return;
  
  int ecode = op->errorCode();
  int i = op->arg(0).findRev( "/" );
  QString sBuff = op->arg(0).right( op->arg(0).length() - i - 1 );

  switch (op->operation()) {
  case QNetworkProtocol::OpMkDir: {
    if (op->state() == QNetworkProtocol::StFailed) {
      QString mess = "Couldn't create directory: ";
      mess = mess + op->arg(0);
      _referens->addConnectionInformation(mess, blue);
    }
    else {
      QString mess = "Successfully created directory: ";
      mess = mess + op->arg(0);
      _referens->addConnectionInformation(mess, blue);
    }	
  } 
    break;
    
  case QNetworkProtocol::OpRemove: {
    if(op->state() == QNetworkProtocol::StFailed) {
      QString mess = "Couldn't delete directory or file: ";
      mess = mess + op->arg(0);
      _referens->addConnectionInformation(mess, blue);
    }
    else {
      QString mess = "Successfully deleted directory or file: ";
      mess = mess + op->arg(0);
      _referens->addConnectionInformation(mess, blue);
    }	
  }
    break;

  case QNetworkProtocol::OpRename: {
    if(op->state() == QNetworkProtocol::StFailed) {
      QString mess = "Couldn't rename dir or file: ";
      mess += op->arg(0);
      _referens->addConnectionInformation(mess, blue);
    }
    else {
      QString mess = "Successfully renamed dir or file: ";
      mess += op->arg(0);
      _referens->addConnectionInformation(mess, blue);
    }
  }
    break;
       
  case QNetworkProtocol::OpListChildren: {
    if(op->state() == QNetworkProtocol::StFailed) {
      QString mess = "Couldn't read dir: ";
      mess += url.path();
      url = oldUrl;
      comboBox->insertItem(url.path(), 0);
      comboBox->setCurrentItem(0);
      _referens->addConnectionInformation(mess, red);
    }
       
    else if(op->state() == QNetworkProtocol::StDone) {
      //_referens->addConnectionInformation(op->protocolDetail(), blue);
    }
    else if(op->state() == QNetworkProtocol::StWaiting)
      _referens->addConnectionInformation(op->protocolDetail(), blue);

    else if(op->state() == QNetworkProtocol::StInProgress)
      _referens->addConnectionInformation(op->protocolDetail(), blue);

    else if(op->state() == QNetworkProtocol::StStopped)
      _referens->addConnectionInformation(op->protocolDetail(), blue);
    
  }
    break;
	
  case QNetworkProtocol::OpPut: {
    if(op->state() == QNetworkProtocol::StDone) {
      otherFw->slotRefresh();
      emit signal_nextToCopy();
    }
    else if(op->state() == QNetworkProtocol::StFailed) {
      _referens->addConnectionInformation(op->protocolDetail(), blue);
      emit signal_nextToCopy();
    }
    else if(op->state() == QNetworkProtocol::StWaiting)
      _referens->addConnectionInformation(op->protocolDetail(), blue);

    else if(op->state() == QNetworkProtocol::StInProgress)
      _referens->addConnectionInformation(op->protocolDetail(), blue);

    else if(op->state() == QNetworkProtocol::StStopped)
      _referens->addConnectionInformation(op->protocolDetail(), blue);    
  }      
    break;
    
  case QNetworkProtocol::OpGet: {
    if(op->state() == QNetworkProtocol::StDone) {
      otherFw->slotRefresh();
      _referens->addConnectionInformation("File successfully up/downloaded", blue);
      emit signal_nextToCopy();
    }
    else if(op->state() == QNetworkProtocol::StFailed) {
      _referens->addConnectionInformation(op->protocolDetail(), red);
      QString errormess = ErrorMess::getErrorCode(ecode);
      emit signalFileError( sBuff, errormess );
      emit signal_nextToCopy();
    }
    else if(op->state() == QNetworkProtocol::StWaiting)
      _referens->addConnectionInformation(op->protocolDetail(), blue);

    else if(op->state() == QNetworkProtocol::StInProgress)
      _referens->addConnectionInformation(op->protocolDetail(), blue);

    else if(op->state() == QNetworkProtocol::StStopped)
      _referens->addConnectionInformation(op->protocolDetail(), blue);
  }      
    break;
  }
}
