//ftpbase.cpp
#include <qapplication.h>
#include <qvariant.h>
#include <qframe.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qsplitter.h>
#include <qtextedit.h>
#include <qtoolbutton.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qaction.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qtoolbar.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qstring.h>
#include <qspinbox.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qstyle.h>

#include "ftpbase.h"
#include "settingsdialog.h"
#include "comboboxloadtext.h"
#include "fileviewwidget.h"
#include "errorandpic.h"
#include "stylepopupmenu.h"
#include "connectioninformationtext.h"
#include "filetransferlistview.h"


FtpBase::FtpBase( QWidget* parent,  const char* name, WFlags fl )
    : QMainWindow( parent, name, fl )
{
    setCaption( trUtf8( "easyFTP by GRP7" ) );
    setCentralWidget( new QWidget( this, "qt_central_widget" ) );
    _ftpBaseLayout = new QVBoxLayout( centralWidget(), 11, 6, "ftpBaseLayout"); 
    
    setupFtpView();
    setupTopInfoView();
    setupBottomInfoView();
    setupMenuBar();

    //**********SERVER************

     connect( _lstServerFiles, SIGNAL( signalTransferInfo( const QString &, int, int ) ), _fileTransferListView, SLOT( slotUppdateItem( const QString &, int, int ) ) );
     connect( _lstServerFiles, SIGNAL( signalNewItem( const QString & ) ), _fileTransferListView, SLOT( slotInsertItem( const QString & ) ) );
     connect( _lstServerFiles, SIGNAL( signalFileError( const QString &, const QString & ) ), _fileTransferListView, SLOT( slotFileError( const QString &, const QString & ) ) );
    
    connect(_fileTransferListView, SIGNAL(signalStopTransfer()), _lstServerFiles, SLOT(slotStopTransfer()));
    
    //********LOCAL***************

    connect( _lstLocalFiles, SIGNAL( signalTransferInfo( const QString &, int, int ) ), _fileTransferListView, SLOT( slotUppdateItem( const QString &, int, int ) ) );    
    connect( _lstLocalFiles, SIGNAL( signalNewItem( const QString & ) ), _fileTransferListView, SLOT( slotInsertItem( const QString & ) ) );
    connect( _lstLocalFiles, SIGNAL( signalFileError( const QString &, const QString & ) ), _fileTransferListView, SLOT( slotFileError( const QString &, const QString & ) ) );

    connect(_fileTransferListView, SIGNAL(signalStopTransfer()), _lstLocalFiles, SLOT(slotStopTransfer()));
}

FtpBase::~FtpBase()
{
}

void FtpBase::setupFtpView()
{
    QHBoxLayout *fileMainFrameLayout = new QHBoxLayout( 0, 0, 6, "_fileMainFrameLayout");

//************* LOKALVIEW ********************
    QFrame *localFileFrame = new QFrame( centralWidget(), "_localFileFrame" );
    localFileFrame->setFrameShape( QFrame::Box );
    localFileFrame->setFrameShadow( QFrame::Sunken );
    QVBoxLayout *localFileFrameLayout = new QVBoxLayout( localFileFrame, 11, 6, "_localFileFrameLayout"); 
    
    _lstLocalFiles = new FileWidget( localFileFrame, "_lstLocalFiles", 0 );
    _lstLocalFiles->setWidgetParent(this);
    
    localFileFrameLayout->addWidget( _lstLocalFiles );
    fileMainFrameLayout->addWidget( localFileFrame );

//************* MIDDLEPART *****************
    QVBoxLayout *fileButtonsLayout = new QVBoxLayout( 0, 0, 6, "_fileButtonsLayout"); 
   
    //********* Spacer ****************
    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    fileButtonsLayout->addItem( spacer );
    //********* Rightbutton ***********    
    _btnRight = new QToolButton( centralWidget(), "_btnRight" );
    QToolTip::add( _btnRight, trUtf8( "Upload selected files" ) );
    _btnRight->setIconSet( QIconSet(QPixmap(Pic::getPic(2))));
    fileButtonsLayout->addWidget( _btnRight );

    //********* Spacer ****************    
    QSpacerItem* spacer_2 = new QSpacerItem( 25, 25, QSizePolicy::Minimum, QSizePolicy::Fixed );
    fileButtonsLayout->addItem( spacer_2 );

    //********* Leftbutton ************
    _btnLeft = new QToolButton( centralWidget(), "_btnLeft" );
    _btnLeft->setIconSet( QIconSet(QPixmap(QPixmap(Pic::getPic(3)))));
    QToolTip::add( _btnLeft, trUtf8( "Download selected files" ) );
    fileButtonsLayout->addWidget( _btnLeft );

    //********* Spacer ****************   
    QSpacerItem* spacer_3 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    fileButtonsLayout->addItem( spacer_3 );
    fileMainFrameLayout->addLayout( fileButtonsLayout );
    
//************* SERVERVIEW ********************
    QFrame *serverFileFrame = new QFrame( centralWidget(), "_serverFileFrame" );
    serverFileFrame->setFrameShape( QFrame::Box );
    serverFileFrame->setFrameShadow( QFrame::Sunken );
    QVBoxLayout *serverFileFrameLayout = new QVBoxLayout( serverFileFrame, 11, 6, "_serverFileFrameLayout"); 
   
    _lstServerFiles = new FileWidget( serverFileFrame, "_lstServerFiles");
    _lstServerFiles->setWidgetParent(this);
    serverFileFrameLayout->addWidget( _lstServerFiles );
    fileMainFrameLayout->addWidget( serverFileFrame );
    
    _lstServerFiles->setOtherFw( _lstLocalFiles ); //Point to localFw
    _lstLocalFiles->setOtherFw( _lstServerFiles ); //Point to remoteFw

//************* MAKE CONNECTIONS **************
    
    QObject::connect( _btnRight, SIGNAL(clicked()), _lstLocalFiles , SLOT(slotUpDownload()) );
    QObject::connect( _btnLeft, SIGNAL(clicked()), _lstServerFiles , SLOT(slotUpDownload()) );

    _ftpBaseLayout->addLayout( fileMainFrameLayout );
}


void FtpBase::setupTopInfoView()
{
  
//************* LEFT INFOPART *****************
    QHBoxLayout *downUpLoadMainLayout = new QHBoxLayout( 0, 0, 6, "_downUpLoadMainLayout"); 

//************* LEFT INFOPART *****************
    
    _fileTransferListView = new FileTransferListView( centralWidget(), "_fileTransferListView"  );
    downUpLoadMainLayout->addWidget( _fileTransferListView );
    _fileTransferListView->setFixedHeight( 125 );
    QToolTip::add(_fileTransferListView, trUtf8("Information for filetransfers"));

//************* RIGHT BUTTONS *****************
    QVBoxLayout *downUpLoadButtonsLayout = new QVBoxLayout( 0, 0, 6, "_downUpLoadButtonsLayout"); 

    //********* Settingsbutton ********
    QToolButton *btnSettings = new QToolButton( centralWidget(), "_btnSettings" );
    btnSettings->setTextLabel( trUtf8( "&Settings" ) );
    QToolTip::add( btnSettings, trUtf8( "Connection settings" ) );
    btnSettings->setUsesTextLabel ( TRUE );
    btnSettings->setIconSet(QIconSet(QPixmap(Pic::getPic(1))));
    btnSettings->setUsesBigPixmap( FALSE );
    downUpLoadButtonsLayout->addWidget( btnSettings );

    //********* Connectbutton *********
    _cmdConnect = new QPushButton( centralWidget(), "_cmdConnect" );
    _cmdConnect->setText( trUtf8( "&Connect" ) );
    _cmdConnect->setEnabled( false );
    //////Andra tipen
    QToolTip::add( _cmdConnect, trUtf8( "No hostname selected" ) );
    downUpLoadButtonsLayout->addWidget( _cmdConnect ); 
   
    //********* DisconnectButton ********
    QPushButton *cmdDisconnect = new QPushButton( centralWidget(), "_cmdDisconnect" );
    cmdDisconnect->setText(trUtf8("&Disconnect"));
    downUpLoadButtonsLayout->addWidget(cmdDisconnect);

    downUpLoadMainLayout->addLayout( downUpLoadButtonsLayout );

//************* MAKE CONNECTIONS **************
    QObject::connect( btnSettings, SIGNAL( clicked() ), this, SLOT( slot_dialog() ) );
    QObject::connect( _cmdConnect, SIGNAL( clicked() ), this, SLOT( slot_connect() ) );
    QObject::connect( cmdDisconnect, SIGNAL(clicked()), _lstServerFiles, SLOT(slotDisconnect()));
    QObject::connect( cmdDisconnect, SIGNAL(clicked()), _fileTransferListView, SLOT(slotClear()));
     
    _ftpBaseLayout->addLayout( downUpLoadMainLayout );
}


void FtpBase::setupBottomInfoView()
{
    QSplitter *connectionInformationSplitter = new QSplitter( centralWidget(), "_connectionInformationSplitter" );
    connectionInformationSplitter->setOrientation( QSplitter::Horizontal );

//************* LEFT INFOPART *****************
    _txtConnectionInformation = new ConnectionInformationText( connectionInformationSplitter, "_txtConnectionInformation" );
    _lstServerFiles->setReferens( _txtConnectionInformation );
    _lstLocalFiles->setReferens( _txtConnectionInformation );
    
//************* RIGHT LABELBOX *****************
    QGroupBox *grpInformation = new QGroupBox( connectionInformationSplitter, "_grpInformation" );
    grpInformation->setTitle( trUtf8( "Information:" ) );
    QToolTip::add( grpInformation, trUtf8( "Information" ) );
    grpInformation->setColumnLayout(0, Qt::Vertical );
    grpInformation->layout()->setSpacing( 6 );
    grpInformation->layout()->setMargin( 11 );
    connectionInformationSplitter->setResizeMode(grpInformation, QSplitter::FollowSizeHint);
    
    QVBoxLayout *grpInformationLayout = new QVBoxLayout( grpInformation->layout() );
    grpInformationLayout->setAlignment( Qt::AlignTop );

    _lblHostName = new QLabel( grpInformation, "_lblHostName" );
    _lblHostName->setText( trUtf8( "Hostname:" ) );
    grpInformationLayout->addWidget( _lblHostName );

    _lblPortNumber = new QLabel( grpInformation, "_lblPortNumber" );
    _lblPortNumber->setText( trUtf8( "Portnumber:" ) );
    grpInformationLayout->addWidget( _lblPortNumber );

    _lblUserName = new QLabel( grpInformation, "_lblUserName" );
    _lblUserName->setText( trUtf8( "Username:" ) );
    grpInformationLayout->addWidget( _lblUserName );

    _ftpBaseLayout->addWidget( connectionInformationSplitter );
}


void FtpBase::setupMenuBar()
{   
    _menubar = new QMenuBar( this, "_menubar" );

    //********* Filemenu && Settings**********
    QPopupMenu *fileMenu = new QPopupMenu( this );
    QAction *openSettings = new QAction( trUtf8( "Ftp settings" ), QIconSet( ( QPixmap( Pic::getPic( 1 ) ) ) ), trUtf8( "&Ftp Settings" ), QKeySequence( trUtf8( "Ctrl+S") ), this, trUtf8( "openSettings" ) );
    openSettings->addTo( fileMenu );
    QAction *fileExitAction = new QAction( trUtf8( "Exit" ), trUtf8( "E&xit" ), QKeySequence( trUtf8( "Ctrl+X" ) ), this, "fileExitAction" );
    fileExitAction->addTo( fileMenu );
    _menubar->insertItem( trUtf8( "&File" ), fileMenu );
    
    //********* Stylesmenu **********
    StylePopupMenu *styleMenu = new StylePopupMenu( this );
    _menubar->insertItem( trUtf8( "S&tyle" ), styleMenu );
    
    //********* Helpmenu **********
    _menubar->insertSeparator();
    QPopupMenu *helpMenu = new QPopupMenu( this ); 
    QAction *aboutFtp = new QAction( trUtf8( "About" ), trUtf8( "&About easyFTP" ), QKeySequence( trUtf8( "F1" ) ), this, "aboutFtp" );
    QAction *aboutQt = new QAction( this, "_aboutQt" );
    aboutQt->setText(trUtf8("About Qt"));
    aboutQt->setMenuText( trUtf8( "About &Qt" ) );
    aboutQt->addTo( helpMenu );
    aboutFtp->addTo( helpMenu );
    _menubar->insertItem( trUtf8( "&Help" ), helpMenu );
            
    //************* MAKE CONNECTIONS **************
    connect( fileExitAction, SIGNAL( activated() ), qApp, SLOT( closeAllWindows() ) );
    connect( openSettings, SIGNAL( activated() ), this, SLOT( slot_dialog() ) );
    connect( aboutFtp, SIGNAL( activated() ), this, SLOT( slot_about() ) );
    connect(aboutQt, SIGNAL(activated()), this, SLOT(slot_aboutQt()));
}


void FtpBase::slot_dialog()
{
  SettingsDialog *s = new SettingsDialog(this, "s", TRUE);
  s->show();
  if( s->exec() )
    {
      ComboBoxLoadText *cmbHost = ( ComboBoxLoadText * ) s->child( "_cmbHostName", "QComboBox" );
      if( cmbHost )
      {
	_strHostName = cmbHost->currentText();
	_lblHostName->setText( trUtf8("Hostname: ") + _strHostName );
	QToolTip::remove( _cmdConnect );
	  if( !_strHostName.isEmpty() )
	    {
	      QToolTip::add( _cmdConnect, "Connect to " + _strHostName ); 
	      _cmdConnect->setEnabled( true );
	    }
	  else
	    {
	      QToolTip::add( _cmdConnect, "No hostname selected" ); 
	      _cmdConnect->setEnabled( false ); 
	    }  
      }

      QSpinBox *spinPort = ( QSpinBox * ) s->child( "_spinPortNumber", "QSpinBox" );
      if( spinPort )
      {
	_intPortNumber = (unsigned int)spinPort->value();
	_lblPortNumber->setText( "Portnumber: "  + QString::number(_intPortNumber, 10 ) );       
      }
      
      QComboBox *cmbUser = ( QComboBox * ) s->child( "_cmbUserName", "QComboBox" );
      if( cmbUser )
      {
	_strUserName = cmbUser->currentText();
	_lblUserName->setText( "Username: " + _strUserName );
      }
      
      QLineEdit *editPass = (QLineEdit*) s->child( "_linePassword", "QLineEdit" );
      if ( editPass )
	
	_strPassword = editPass->text();
 
    }
    delete s;
}


void FtpBase::slot_about()
{
  QMessageBox::information( this, "About easyFTP", "\tThis is the easyFTP application\n\tCopyright 2002 by\n\n\tJoe Paulsson\t\t<joepaulsson@hotmail.com>\t\n\tAnders Pettersson\t<d00_ape@hotmail.com>\n\tJesper Miller\t\t<miller@linux.nu>\n\tDavid Edstrm\t\t<fallen@mangakai.org>\n\tMattias Gustafsson\t<matteg@linux.nu>");  
}

void FtpBase::slot_aboutQt()
{
    QMessageBox::aboutQt( this, "Qt" );
}

void FtpBase::slot_connect()
{
  _lstServerFiles->connectToFtp( _strHostName, _strUserName, _strPassword,
_intPortNumber );   
}

void FtpBase::changeBtnState(bool st) {
  _btnRight->setEnabled(st);
  _btnLeft->setEnabled(st);
}
    
      
