//settingsdialog.cpp

#include <qvariant.h>
#include <qcombobox.h>
#include <qframe.h>
#include <qheader.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qmime.h>
#include <qdragobject.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qdir.h>

#include "settingsdialog.h"
#include "comboboxloadtext.h"
#include "favouriteslistview.h"

SettingsDialog::SettingsDialog( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "SettingsDialog" );
    resize( 446, 381 ); 
    setCaption( trUtf8( "Ftp - settings" ) );
    
    _settingsDialogLayout = new QVBoxLayout( this, 11, 6, "SettingsDialogLayout"); 

    setupDir();
    setupFavourites();
    setupInputFields();

    // signals and slots connections
    connect( _cmdOK, SIGNAL( clicked() ), this, SLOT( slot_OK() ) );
    connect( _cmdCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
    connect( _cmdAddGenre, SIGNAL( clicked() ), this, SLOT( slot_addGenre() ) );
    connect( _cmdAddHostName, SIGNAL( clicked() ), this, SLOT( slot_addHostName() ) );
    connect( _cmdErase, SIGNAL( clicked() ), this, SLOT( slot_erase() ) );
}

void SettingsDialog::setupFavourites()
{

  _favouritesFrame = new QFrame( this, "_favouritesFrame" );
  _favouritesFrame->setFrameShape( QFrame::Box );
  _favouritesFrame->setFrameShadow( QFrame::Sunken );
  _favouritesFrameLayout = new QHBoxLayout( _favouritesFrame, 11, 6, "_favouritesFrameLayout"); 

  _listFavourites = new FavouritesListView( _favouritesFrame, "_listFavourites" );
  _listFavourites->setPath( getPath() );
  _listFavourites->setReferens( this );
  _listFavourites->load();
  
  _favouritesFrameLayout->addWidget( _listFavourites );
  
  _favouritesButtonsLayout = new QVBoxLayout( 0, 0, 6, "_favouritesButtonsLayout"); 
  QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
  _favouritesButtonsLayout->addItem( spacer );
  
  _cmdAddGenre = new QPushButton( _favouritesFrame, "_cmdAddGenre" );
  _cmdAddGenre->setText( trUtf8( "&Add genre" ) );
  _favouritesButtonsLayout->addWidget( _cmdAddGenre );
  QSpacerItem* spacer_2 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
  _favouritesButtonsLayout->addItem( spacer_2 );
  
  _cmdAddHostName = new QPushButton( _favouritesFrame, "_cmdAddHostName" );
  _cmdAddHostName->setText( trUtf8( "Add hos&tname" ) );
  _favouritesButtonsLayout->addWidget( _cmdAddHostName );
  QSpacerItem* spacer_3 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
  _favouritesButtonsLayout->addItem( spacer_3 );

  _cmdErase = new QPushButton( _favouritesFrame, "_cmdErase" );
  _cmdErase->setText( trUtf8( "&Erase" ) );
  _favouritesButtonsLayout->addWidget( _cmdErase );
  QSpacerItem* spacer_4 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
  _favouritesButtonsLayout->addItem( spacer_4 );
  _favouritesFrameLayout->addLayout( _favouritesButtonsLayout );
  _settingsDialogLayout->addWidget( _favouritesFrame );

}

void SettingsDialog::setupInputFields()
{
    _addressFrame = new QFrame( this, "_addressFrame" );
    _addressFrame->setFrameShape( QFrame::Box );
    _addressFrame->setFrameShadow( QFrame::Sunken );
    _addressFrameLayout = new QVBoxLayout( _addressFrame, 11, 6, "_addressFrameLayout"); 

    _addressInfoLayout = new QHBoxLayout( 0, 0, 6, "_addressInfoLayout"); 

    _addressInfoLabelLayout = new QVBoxLayout( 0, 0, 6, "_addressInfoLabelLayout"); 

    _lblHostName = new QLabel( _addressFrame, "_lblHostName" );
    _lblHostName->setText( trUtf8( "&Hostname:" ) );
    _addressInfoLabelLayout->addWidget( _lblHostName );

    _lblPortNumber = new QLabel( _addressFrame, "_lblPortNumber" );
    _lblPortNumber->setText( trUtf8( "Port &number:" ) );
    _addressInfoLabelLayout->addWidget( _lblPortNumber );

    _lblUserName = new QLabel( _addressFrame, "_lblUserName" );
    _lblUserName->setText( trUtf8( "&Username:" ) );
    _addressInfoLabelLayout->addWidget( _lblUserName );

    _lblPassword = new QLabel( _addressFrame, "_lblPassword" );
    _lblPassword->setText( trUtf8( "&Password:" ) );
    _addressInfoLabelLayout->addWidget( _lblPassword );
    _addressInfoLayout->addLayout( _addressInfoLabelLayout );

    _addressInfoInputLayout = new QVBoxLayout( 0, 0, 6, "_addressInfoInputLayout"); 
    
    _cmbHostName = new ComboBoxLoadText( FALSE, _addressFrame, "_cmbHostName");
    _cmbHostName->setPath( getPath() );
    _cmbHostName->load();
    _lblHostName->setBuddy( _cmbHostName );
    _addressInfoInputLayout->addWidget( _cmbHostName );

    _spinPortNumber = new QSpinBox( _addressFrame, "_spinPortNumber" );
    _spinPortNumber->setMaxValue( 65565 );
    _spinPortNumber->setValue( 21 );
    _lblPortNumber->setBuddy( _spinPortNumber );
    _addressInfoInputLayout->addWidget( _spinPortNumber );

    _cmbUserName = new QComboBox( FALSE, _addressFrame, "_cmbUserName" );
    _cmbUserName->setEditable( TRUE );
    _lblUserName->setBuddy( _cmbUserName );
    _addressInfoInputLayout->addWidget( _cmbUserName );

    _linePassword = new QLineEdit( _addressFrame, "_linePassword" );
    _linePassword->setEchoMode( QLineEdit::Password );
    _lblPassword->setBuddy( _linePassword );
    _addressInfoInputLayout->addWidget( _linePassword );
    _addressInfoLayout->addLayout( _addressInfoInputLayout );
    _addressFrameLayout->addLayout( _addressInfoLayout );

    _addressButtonsLayout = new QHBoxLayout( 0, 0, 6, "_addressButtonsLayout"); 
    QSpacerItem* spacer_5 = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
    _addressButtonsLayout->addItem( spacer_5 );

    _cmdOK = new QPushButton( _addressFrame, "_cmdOK" );
    _cmdOK->setText( trUtf8( "&OK" ) );
    _addressButtonsLayout->addWidget( _cmdOK );

    _cmdCancel = new QPushButton( _addressFrame, "_cmdCancel" );
    _cmdCancel->setText( trUtf8( "&Cancel" ) );
    _addressButtonsLayout->addWidget( _cmdCancel );
    _addressFrameLayout->addLayout( _addressButtonsLayout );
    _settingsDialogLayout->addWidget( _addressFrame );
    _cmdOK->setDefault(true);
}

SettingsDialog::~SettingsDialog()
{
}

void SettingsDialog::setupDir()
{
  QString tmp = QDir::homeDirPath() + trUtf8( "/.easyFTP" );
  QDir *dir = new QDir( tmp );
  if( !dir->exists() )
    dir->mkdir( tmp );
  _path = tmp + "/";
}

QString SettingsDialog::getPath()
{
  return _path;
}

void SettingsDialog::slot_addGenre()
{
  _listFavourites->addGenre();
}

void SettingsDialog::slot_addHostName()
{
  _listFavourites->addHostName();
}

void SettingsDialog::slot_erase()
{
  _listFavourites->erase();
}

void SettingsDialog::slot_OK()
{
  _listFavourites->save();
  _cmbHostName->save();
  accept();
}

void SettingsDialog::slot_displayInfo( QListViewItem *i )
{
  
  if ( i->childCount() == 0 )
    {
      _cmbHostName->setCurrentText( i->text(1) );
      _cmbUserName->setCurrentText( i->text(2) ); 
      bool f = true;
      _spinPortNumber->setValue( i->text(3).toInt( &f, 10 ) );
      _linePassword->setFocus();
    } 
  
}
