// stylepopupmenu.h
#include <qfont.h>
#include <qpopupmenu.h> 
#include <qaction.h>
#include <qmenubar.h>
#include <qapplication.h>
#include <qpalette.h>

#include "stylepopupmenu.h"
#include "metal.h"
#include "wood.h"

StylePopupMenu::StylePopupMenu( QWidget *parent, const char *name ) : QPopupMenu( parent, name )
{
  
  _appFont = QApplication::font();

  QAction *_windowsStyleAction = new QAction( this, "_windowsStyleAction" );
  _windowsStyleAction->setText( trUtf8( "Windows" ) );
  _windowsStyleAction->setMenuText( trUtf8( "&Windows" ) );
  _windowsStyleAction->addTo( this );
  connect( _windowsStyleAction, SIGNAL( activated() ), this, SLOT( slot_setWindowsStyle() ) );

  QAction *_platinumStyleAction = new QAction( this, "_platinumStyleAction" );
  _platinumStyleAction->setText( trUtf8( "Platinum" ) );
  _platinumStyleAction->setMenuText( trUtf8( "&Platinum" ) );
  _platinumStyleAction->addTo( this );
  connect( _platinumStyleAction, SIGNAL( activated() ), this, SLOT( slot_setPlatinumStyle() ) );

  QAction *_CDEStyleAction = new QAction( this, "_CDEStyleAction" );
  _CDEStyleAction->setText( trUtf8( "CDE" ) );
  _CDEStyleAction->setMenuText( trUtf8( "&CDE" ) );
  _CDEStyleAction->addTo( this );
  connect( _CDEStyleAction, SIGNAL( activated() ), this, SLOT( slot_setCDEStyle() ) );

  QAction *_motifStyleAction = new QAction( this, "_motifStyleAction" );
  _motifStyleAction->setText( trUtf8( "Motif" ) );
  _motifStyleAction->setMenuText( trUtf8( "&Motif" ) );
  _motifStyleAction->addTo( this );
  connect( _motifStyleAction, SIGNAL( activated() ), this, SLOT( slot_setMotifStyle() ) );

  QAction *_sgiStyleAction = new QAction( this, "_sgiStyleAction" );
  _sgiStyleAction->setText( trUtf8( "Sgi" ) );
  _sgiStyleAction->setMenuText( trUtf8( "&Sgi" ) );
  _sgiStyleAction->addTo( this );
  connect( _sgiStyleAction, SIGNAL( activated() ), this, SLOT( slot_setSgiStyle() ) );

  QAction *_woodStyleAction = new QAction( this, "_woodStyleAction" );
  _woodStyleAction->setText( trUtf8( "Wood" ) );
  _woodStyleAction->setMenuText( trUtf8( "&Wood" ) );
  _woodStyleAction->addTo( this );
  connect( _woodStyleAction, SIGNAL( activated() ), this, SLOT( slot_setWoodStyle() ) );

  QAction *_stoneStyleAction = new QAction( this, "_stoneStyleAction" );
  _stoneStyleAction->setText( trUtf8( "Stone" ) );
  _stoneStyleAction->setMenuText( trUtf8( "&Stone" ) );
  _stoneStyleAction->addTo( this );
  connect( _stoneStyleAction, SIGNAL( activated() ), this, SLOT( slot_setStoneStyle() ) );

}


void StylePopupMenu::slot_setWindowsStyle()
{
  qApp->setStyle("Windows");
  qApp->setFont( _appFont, TRUE );
}

void StylePopupMenu::slot_setSgiStyle()
{
  qApp->setStyle("Sgi");
  qApp->setFont( _appFont, TRUE );
}

void StylePopupMenu::slot_setPlatinumStyle()
{  
  qApp->setStyle("Platinum");
  QPalette p( QColor( 239, 239, 239 ) );
  qApp->setPalette( p, TRUE );
  qApp->setFont( _appFont, TRUE );
}


void StylePopupMenu::slot_setCDEStyle()
{
  QPalette p( QColor( 75, 123, 130 ) );
  p.setColor( QPalette::Active, QColorGroup::Base, QColor( 55, 77, 78 ) );
  p.setColor( QPalette::Inactive, QColorGroup::Base, QColor( 55, 77, 78 ) );
  p.setColor( QPalette::Disabled, QColorGroup::Base, QColor( 55, 77, 78 ) );
  p.setColor( QPalette::Active, QColorGroup::Highlight, Qt::white );
  p.setColor( QPalette::Active, QColorGroup::HighlightedText, QColor( 55, 77, 78 ) );
  p.setColor( QPalette::Inactive, QColorGroup::Highlight, Qt::white );
  p.setColor( QPalette::Inactive, QColorGroup::HighlightedText, QColor( 55, 77, 78 ) );
  p.setColor( QPalette::Disabled, QColorGroup::Highlight, Qt::white );
  p.setColor( QPalette::Disabled, QColorGroup::HighlightedText, QColor( 55, 77, 78 ) );
  p.setColor( QPalette::Active, QColorGroup::Foreground, Qt::white );
  p.setColor( QPalette::Active, QColorGroup::Text, Qt::white );
  p.setColor( QPalette::Active, QColorGroup::ButtonText, Qt::white );
  p.setColor( QPalette::Inactive, QColorGroup::Foreground, Qt::white );
  p.setColor( QPalette::Inactive, QColorGroup::Text, Qt::white );
  p.setColor( QPalette::Inactive, QColorGroup::ButtonText, Qt::white );
  p.setColor( QPalette::Disabled, QColorGroup::Foreground, Qt::lightGray );
  p.setColor( QPalette::Disabled, QColorGroup::Text, Qt::lightGray );
  p.setColor( QPalette::Disabled, QColorGroup::ButtonText, Qt::lightGray );
  qApp->setPalette( p, TRUE );
  qApp->setFont( QFont( "times", _appFont.pointSize() ), TRUE );
} 

void StylePopupMenu::slot_setMotifStyle()
{
  qApp->setStyle("Motif");
  QPalette p( QColor( 192, 192, 192 ) );
  qApp->setPalette( p, TRUE );
}

void StylePopupMenu::slot_setWoodStyle()
{
  qApp->setStyle( new NorwegianWoodStyle );
  qApp->setFont( _appFont, TRUE );
}

void StylePopupMenu::slot_setStoneStyle()
{
  qApp->setStyle( new MetalStyle );
  qApp->setFont( _appFont, TRUE );
}
